"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

from __future__ import absolute_import

import email.parser
import io
import logging
import os
import re

import vcsinfo


LOGGER = logging.getLogger(__name__)


class VCSPyInfo(vcsinfo.VCS):
    """Minimal class for querying Python info files"""

    VERSION_RE = re.compile(r'(?P<coarse>[.\d]*\d)\.(?P<number>\d+)(.dev(?P<modified>\d+))?')

    def __init__(self, dirname):
        vcsinfo.VCS.__init__(self)

        # Look for a specific .egg-info subdirectory which will contain more information
        # than just a PKG-INFO in dirname.
        files = os.listdir(dirname)
        contents = [ent for ent in files if ent.endswith('.egg-info')]
        if contents:
            LOGGER.debug(f'Found {contents}')
            _dirname = os.path.join(dirname, contents[0])
            LOGGER.debug(f'Picking {_dirname}')
        else:
            # Didn't find a .egg-info - stick with dirname
            _dirname = dirname
        self.detect_source_root(_dirname)
        LOGGER.debug(f'Matched {self.vcs}: {dirname}')


    def detect_source_root(self, dirname):
        '''Identify if a directory might be an .info-info directory.'''

        dirname = os.path.realpath(dirname)
        try:
            pi_path = os.path.join(dirname, 'PKG-INFO')
            LOGGER.debug(f'Reading {pi_path}')
            with io.open(pi_path, encoding='utf-8', errors='replace') as piobj:
                raw_pi = piobj.read()
            self.pkg_info = email.parser.Parser().parsestr(raw_pi)
        except (ValueError, IOError) as err:
            raise TypeError(f'Directory "{dirname}" is not an pkg_info directory: {err}') from err
        self.source_root = dirname
        sources_txt = os.path.join(dirname, 'SOURCES.txt')
        try:
            with io.open(sources_txt, encoding='utf-8', errors='replace') as st_obj:
                self.files = [line.rstrip(os.linesep) for line in st_obj.readlines()]
        except IOError as err:
            raise TypeError(f'Failed reading {sources_txt}: {err}') from err


    def _get_version(self):
        return self.pkg_info.get('Version')


    def _match_version(self):
        try:
            mobj = self._ver_mobj
        except AttributeError:
            # pylint: disable=attribute-defined-outside-init
            mobj = self._ver_mobj = re.match(self.VERSION_RE, self._get_version())
        return mobj


    @property
    def name(self):
        return self.pkg_info.get('Name')


    @property
    def branch(self):
        mobj = self._match_version()
        if mobj:
            branch = mobj.groupdict().get('coarse', '0') or '0'
        else:
            branch = '0'
        return branch


    @property
    def user(self):
        return None


    @property
    def id(self):
        mobj = self._match_version()
        if mobj:
            ids = '.'.join((
                mobj.groupdict().get('coarse', '0') or '0',
                mobj.groupdict().get('number', '0') or '0',
            ))
        else:
            ids = '0'
        return ids


    @property
    def id_short(self):
        return self.id


    @property
    def number(self):
        mobj = self._match_version()
        if mobj:
            number = mobj.groupdict().get('number', '0') or '0'
        else:
            number = '0'
        return int(number)


    def list_files(self):
        return self.files


    @property
    def modified(self):
        mobj = re.match(self.VERSION_RE, self.id)
        if mobj:
            mod = mobj.groupdict().get('modified', '0') or '0'
        else:
            mod = '0'
        return int(mod)


VCS = VCSPyInfo


# Local Variables:
# fill-column: 100
# End:



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

from __future__ import absolute_import

import logging
import os
import vcsinfo

try:
    import git
except ImportError as err:
    # pylint: disable=C0301
    raise vcsinfo.VCSUnsupported(f'GIT VCS module requires GitPython: {err}')


LOGGER = logging.getLogger(__name__)


class VCSGit(vcsinfo.VCS):
    """
    Class used to retrieve information about a Git managed source tree.
    """

    git_to_vcsinfo_status = {
        'M': vcsinfo.ST_MOD,
        'A': vcsinfo.ST_ADD,
        'D': vcsinfo.ST_REM,
        # don't know how the below maps
        # '' : vcsinfo.ST_DEL,
        # '' : vcsinfo.ST_UNK,
        # '' : vcsinfo.ST_IGN,
        # '' : vcsinfo.ST_CLN,
    }

    def __init__(self, dirname):
        """Constructor"""
        vcsinfo.VCS.__init__(self)
        try:
            # pylint: disable=E1101
            self.vcs_obj = git.Repo(dirname)
        except git.exc.InvalidGitRepositoryError as exc:  # pylint: disable=E1101
            raise TypeError(f"Directory '{dirname}' not managed by {self.vcs}") from exc
        try:
            self.source_root = self.vcs_obj.working_tree_dir
            # The above may or may not raise an AssertionError
            if not self.source_root:
                # Here is another reason to raise an AssertionError
                raise AssertionError("No git working_tree_dir")
        except AssertionError as exc:
            raise TypeError(f"Directory '{dirname}' not a working {self.vcs} checkout") from exc

        # make sure the git working tree is the same as our directory
        if os.path.abspath(self.source_root) != os.path.abspath(dirname):
            raise TypeError(
                f"Directory '{dirname}' is managed by {self.vcs}, but is not the root of the repository"
            )
        LOGGER.debug(f'Matched {self.vcs}: {dirname}')

    @property
    def upstream_repo(self):
        """The location of the upstream VCS repository."""
        try:
            return self.vcs_obj.remotes.upstream.url
        except AttributeError:
            try:
                return self.vcs_obj.remotes.origin.url
            except AttributeError:
                return None

    @property
    def name(self):
        if not self._name:
            if self.vcs_obj.remotes:
                # There's an upstream - use the basename for the name
                path = self.vcs_obj.remotes.origin.url
            else:
                path = self.source_root
            self._name = os.path.splitext(os.path.basename(path))[0]
        return self._name

    @property
    # pylint: disable=R0912
    def branch(self):
        found_branch = 'DETACHED'

        try:
            found_branch = self.vcs_obj.active_branch.name
        except TypeError as exc:
            commits = {}
            branches = {}
            master_branch = None

            for branch in self.vcs_obj.branches:
                if branch.name == 'master':
                    master_branch = branch
                else:
                    branches[branch.name] = branch

            def set_branch_info(_branch, _git_obj):
                """registers a commit"""
                if _git_obj.hexsha in commits:
                    return False
                commits[_git_obj.hexsha] = {
                    'branch': _branch,
                    'object': _git_obj,
                }
                return True

            class BranchDiscoveryDone(Exception):
                """Error signifying end of branch discovery."""

            try:
                # Prime commit with 'master' branch first
                for branch in [master_branch] + list(branches.values()):
                    # If the branch is None continue (if we didn't find a
                    # master branch, for instance)
                    if not branch:
                        continue
                    # The reference isn't at the head of a branch else it
                    # would be .active_branch.name
                    if not set_branch_info(branch.name, branch.object):
                        # This object is already on another branch
                        continue
                    # Cannot follow branch name through a merge
                    if len(branch.object.parents) > 1:
                        continue
                    for git_obj in branch.object.iter_parents():
                        if git_obj.hexsha == self.vcs_obj.head.object.hexsha:
                            found_branch = branch.name
                            raise BranchDiscoveryDone() from exc
                        if not set_branch_info(branch.name, git_obj):
                            # This object is already on another branch
                            break
                        if len(git_obj.parents) > 1:
                            # Cannot follow branch name through a merge
                            break

            except BranchDiscoveryDone:
                pass

        return found_branch

    @property
    def user(self):
        user = ''
        if self.vcs_obj.heads:
            user = self.vcs_obj.rev_parse(
                self.vcs_obj.head.name
            ).committer.email
        return user.replace('@adobe.com', '')

    @property
    def id(self):
        return self.vcs_obj.rev_parse(self.vcs_obj.head.name).hexsha

    @property
    def id_short(self):
        return self.id[:6]

    @property
    def number(self):
        """
        Simulate the commit number.  This is done by looking at the commit
        offset from the base of the tree with respect to the branch where
        the commit is found.  This will return a number that should be
        unique to the branch and fairly stable across future commits
        (provided a branch is not rebased).
        """
        number = 0
        if not self.vcs_obj.heads:
            # There are no commits in this tree
            return number

        commit_id = self.id
        branch_name = self.branch
        try:
            if hasattr(self.vcs_obj.branches, branch_name):
                branch = getattr(self.vcs_obj.branches, branch_name)
            else:
                branch = self.vcs_obj.head
            # The below "sum(1 . . .)" counts the "length" of the
            # iter_parents() generator - so branch_count is the tree height
            # of the branch object (which is a HEAD).
            branch_count = 1 + sum(1 for _ in branch.object.iter_parents())

            # Branches are only HEADs
            if commit_id == branch.object.hexsha:
                # The commit is the HEAD of the branch.
                number = branch_count
            else:
                # The commit is not the HEAD, but we want the commit number
                # so step back how far below the head it is.
                for git_obj in branch.object.iter_parents():
                    branch_count -= 1
                    if git_obj.hexsha == commit_id:
                        number = branch_count
                        break
                else:
                    # The commit wasn't found on the branch line where it
                    # is located - unpossible!
                    raise Exception(
                        f"Internal error: commit {commit_id} not on commit's branch {branch_name}"
                    )
        except TypeError:
            pass

        return number

    def status(self):
        status = [[], [], [], [], [], [], []]

        for line in self.vcs_obj.git.status(porcelain=True).split('\n'):
            if not line:
                continue
            (change, filename) = line.split()[0:2]
            if change in self.git_to_vcsinfo_status:
                status[self.git_to_vcsinfo_status[change]].append(filename)

        # pylint: disable=R1718
        vcs_files = set([
            fn
            for fn
            in self.vcs_obj.git.ls_files().split('\n')
            if fn
        ])

        clean_files = set(vcs_files)
        clean_files -= set(status[vcsinfo.ST_MOD])
        clean_files -= set(status[vcsinfo.ST_ADD])
        clean_files -= set(status[vcsinfo.ST_REM])
        clean_files = list(clean_files)
        clean_files.sort()
        status[vcsinfo.ST_CLN] = clean_files

        return status

    def list_files(self):
        status = self.status()
        vcs_files = list(
            set(status[vcsinfo.ST_CLN])
            | set(status[vcsinfo.ST_ADD])
            | set(status[vcsinfo.ST_MOD])
        )
        vcs_files.sort()
        return vcs_files


VCS = VCSGit



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

from __future__ import absolute_import

import logging
import os
import vcsinfo

try:
    import P4
except ImportError as err:
    raise vcsinfo.VCSUnsupported(
        "Perforce VCS module requires the P4Python library to be installed."
        # pylint: disable=C0301
        f'  See http://www.perforce.com/perforce/doc.current/manuals/p4script/03_python.html for more details: {err}'
    )


LOGGER = logging.getLogger(__name__)


class VCSPerforce(vcsinfo.VCS):
    """
    Class used to retrieve information about a Perforce managed source tree.
    """
    # pylint: disable=too-many-instance-attributes

    p4_to_vcs_status = {
        'edit': vcsinfo.ST_MOD,
        'integrate': vcsinfo.ST_MOD,
        'add': vcsinfo.ST_ADD,
        'branch': vcsinfo.ST_MOD,
        'move/add': vcsinfo.ST_ADD,
        'delete': vcsinfo.ST_REM,
        'move/delete': vcsinfo.ST_REM,
        # don't know how the below maps
        # ''      : vcsinfo.ST_DEL,
        # ''      : vcsinfo.ST_UNK,
        # ''      : vcsinfo.ST_IGN,
        # ''      : vcsinfo.ST_CLN,
    }

    def __init__(
            self,
            dirname,
            development_path='trunk',
            branches_path='branches',
    ):
        """Constructor"""
        vcsinfo.VCS.__init__(self)
        self._development_path_id = development_path
        self._branches_path_id = branches_path
        self._depot_root = ''

        self.detect_source_root(dirname)
        self.vcs_obj = P4.P4(cwd=self.source_root)
        self.vcs_obj.connect()
        self.client = self.vcs_obj.fetch_client()
        self._map = P4.Map(self.client['View'])
        self._inv_map = self._map.reverse()
        self._branch = None
        LOGGER.debug(f'Matched {self.vcs}: {dirname}')

    def __del__(self):
        """
        Make sure the perforce connection is closed when this object is removed
        """
        if hasattr(self, 'vcs_obj') and self.vcs_obj:
            self.vcs_obj.disconnect()

    def detect_source_root(self, directory):
        """
        Traverse directories to find the perforce source root.
        """
        vcs_obj = None
        try:
            real_directory = os.path.realpath(directory)
            vcs_obj = P4.P4(cwd=real_directory)
            vcs_obj.connect()

            p4_probe = real_directory
            if os.path.isdir(real_directory):
                # Add the '...' because the actual directory may not be
                # mapped, but its contents may be mapped.
                p4_probe = os.path.sep.join((real_directory, '...'))

            mapping = vcs_obj.run("where", p4_probe)[0]

            path_dirs = mapping['path'].split('/')
            depot_path_dirs = mapping['depotFile'].split('/')
            if self._development_path_id in depot_path_dirs:
                dev_path_index = depot_path_dirs.index(
                    self._development_path_id
                )
                pop_idx = dev_path_index - len(depot_path_dirs) + 1
            elif self._branches_path_id in depot_path_dirs:
                branches_path_index = depot_path_dirs.index(
                    self._branches_path_id
                )
                pop_idx = branches_path_index - len(depot_path_dirs) + 2
            else:
                raise TypeError(f"Directory '{real_directory}' not managed by p4 client {self.client}")

            self.source_root = '/'.join(path_dirs[:pop_idx])
            self._depot_root = '/'.join(depot_path_dirs[:pop_idx])

        except P4.P4Exception as exc:
            raise TypeError(f"Directory '{real_directory}' is not managed by p4: {exc}") from exc
        finally:
            if vcs_obj and vcs_obj.connected():
                vcs_obj.disconnect()

    @property
    def upstream_repo(self):
        return self._depot_root

    @property
    def name(self):
        if not self._name:
            dirs = self._depot_root.split('/')
            if self._development_path_id == dirs[-1]:
                self._name = dirs[-2]
            elif self._branches_path_id == dirs[-2]:
                self._name = dirs[-3]
        return self._name

    @property
    def branch(self):
        if not self._branch:
            self._branch = self._depot_root.rsplit('/', maxsplit=1)[-1]
        return self._branch

    @property
    def id(self):
        changes = self.vcs_obj.run(
            "changes",
            "-s",
            "submitted",
            "-m",
            "1",
            self.source_root + '/...',
        )
        vcs_id = None
        if changes:
            vcs_id = changes[0]['change']
        return vcs_id

    @property
    def user(self):
        changes = self.vcs_obj.run(
            "changes",
            "-s",
            "submitted",
            "-m",
            "1",
            self.source_root + '/...',
        )
        vcs_id = None
        if changes:
            vcs_id = changes[0]['user']
        return vcs_id

    def classify_status(self, info_list, status_func):
        """Sort files into status lists."""
        status = ([], [], [], [], [], [], [])

        for info in info_list:
            status[self.p4_to_vcs_status[status_func(info)]].append(info)

        return status

    def status(self):
        status = ([], [], [], [], [], [], [])

        open_file_info = {
            ofi['clientFile']: ofi
            for ofi
            in self.vcs_obj.run("opened", self.source_root + '/...')
        }

        file_map = {
            fmap['clientFile']: fmap
            for fmap
            in self.vcs_obj.run("where", *open_file_info.keys())
        }

        for client_file, change_info in open_file_info.items():
            # pylint: disable=C0301
            source_file = file_map[client_file]['path'][len(self.source_root) + 1:]
            status[self.p4_to_vcs_status[change_info['action']]].append(
                source_file
            )

        return status

    def list_files(self):
        status = ([], [], [], [], [], [], [])

        depot_file_info = {
            ofi['depotFile']: ofi
            for ofi
            in self.vcs_obj.run("files", self.source_root + '/...')
        }

        file_map = {
            fmap['depotFile']: fmap
            for fmap
            in self.vcs_obj.run("where", *depot_file_info.keys())
        }

        for depot_file, change_info in depot_file_info.items():
            # pylint: disable=C0301
            source_file = file_map[depot_file]['path'][len(self.source_root) + 1:]
            status[self.p4_to_vcs_status[change_info['action']]].append(
                source_file
            )

        cur_changeset = self.status()
        source_tree_files = list(
            (set(status[vcsinfo.ST_ADD])
             | set(status[vcsinfo.ST_MOD])
             | set(status[vcsinfo.ST_CLN])
             | set(cur_changeset[vcsinfo.ST_ADD]))
            - set(cur_changeset[vcsinfo.ST_REM])
        )
        source_tree_files.sort()

        return source_tree_files


VCS = VCSPerforce



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

from __future__ import print_function

import glob
import logging
import os
import sys


LOGGER = logging.getLogger(__file__)


class VCSUnsupported(Exception):
    """Custom error signifying a specific VCS is not supported"""


def search_parent_dirs(directory, filename):
    """
    Search a directory and each parent directory for a filename.
    If found return the full path; otherwise return ''.
    """

    found = ''

    dir_parts = directory.split(os.path.sep)
    while dir_parts:
        filename_parts = dir_parts + [filename]
        filename_path = os.path.sep.join(filename_parts)
        if os.path.exists(filename_path):
            found = filename_path
            break
        dir_parts.pop()

    return found


def last_mtime(files, prefix=None):
    """
    Find the last modified time for a given list of files.
    """
    mtime = None

    if files:
        if prefix:
            _files = [os.path.join(prefix, f) for f in files]
        else:
            _files = files

        # NOTE: If the file doesn't exist then return 0, but then make sure a
        # 0 result gets mapped back to None.
        mtime = int(max([
            os.path.exists(fn) and os.stat(fn).st_mtime or 0 for fn in _files
        ])) or None

    return mtime


ST_MOD = 0
ST_ADD = 1
ST_REM = 2
ST_DEL = 3
ST_UNK = 4
ST_IGN = 5
ST_CLN = 6


class VCS:
    """Base class for a source tree managed by VCS"""

    def __init__(self):
        """Base constructor"""
        # Automatically set the VCS engine name
        self.vcs = self.__class__.__name__
        if self.vcs.startswith('VCS'):
            self.vcs = self.vcs[3:].lower()

        self.source_root = ''
        self._name = ''
        self._upstream_repo = ''

    @property
    def upstream_repo(self):
        """The location of the up-stream VCS repository."""
        return self._upstream_repo

    @property
    def name(self):
        """The name of the VCS project."""
        return self._name

    @property
    # pylint: disable=R0201
    def branch(self):
        """The branch name."""
        return None

    @property
    # pylint: disable=R0201,invalid-name
    def id(self):
        """The native commit identification."""
        return None

    @property
    # pylint: disable=R0201
    def user(self):
        """The user who performed the commit (if applicable)."""
        return 'n/a'

    @property
    def id_short(self):
        """
        Differs from .id for VCS that have large hash values that
        can be abbreviated.
        """
        return self.id

    @property
    def number(self):
        """
        This is the same as .id if commits have increasing commit numbers.
        Otherwise this simulates an increasing commit number.
        """
        return int(self.id)

    @property
    def modified(self):
        """
        Return none if there are no local modifications to the VCS checkout,
        otherwise return epoc seconds of the most recently modified file.

        NOTE: modified will update each time a file in the changeset is
        modified.  It cannot update, however, when a REMOVED file is added
        to the changeset because there may no longer be a file in the
        file system to stat() to get the removed time!  This means that
        modified can only updated with MODIFIED and ADDED files.
        """
        return last_mtime(
            sum(self.status()[ST_MOD:ST_ADD + 1], []),
            prefix=self.source_root
        ) or 0

    @property
    def release(self):
        """
        String representing the current state of the branch, including the
        commit number, id_short, and whether any modifications have been made
        to the local tree.
        """
        modified = self.modified
        modstr = f'.M{modified}'

        rel_str = f'{self.number}.I{self.id_short}{modstr}'

        return rel_str

    @property
    def id_string(self):
        """
        String providing a unique id of the branch and release.
        """
        return str(self.branch) + "-" + str(self.release)

    # pylint: disable=R0201
    def status(self):
        """
        Returns a tuple of lists of modified files in the changeset:
        (
            [MODIFIED, ...], # ST_MOD: changeset is modifying files
            [ADDED, ...],    # ST_ADD: changeset is adding file
            [REMOVED, ...],  # ST_REM: changeset is removing file
            [DELETED, ...],  # ST_DEL: file is missing but not in changeset
            [UNKNOWN, ...],  # ST_UNK: file exists but not in changeset
            [IGNORED, ...],  # ST_IGN: file exists but VCS explicitly ignores
            [CLEAN, ...],    # ST_CLN: file tracked by VCS and unmodified
        )
        """
        return [], [], [], [], [], [], []

    def list_files(self):
        """
        Return a list of files known to the VCS
        (committed + changeset(added, moved) - changeset(deleted))
        """
        raise NotImplementedError(f'VCS module {self.vcs} must implement list_files()')

    def info(self, include_files=False):
        """
        Return vcs information as a dict, optionally including full file
        information.
        """
        info = {
            'type': self.vcs,
            'upstream_repo': self.upstream_repo,
            'name': self.name,
            'branch': self.branch,
            'id': self.id,
            'id_short': self.id_short,
            'id_string': self.id_string,
            'number': self.number,
            'user': self.user,
            'release': self.release,
        }

        if include_files:
            info['files'] = self.list_files()
            # Do not report UNKNOWN or IGNORED files
            status_info = list(self.status())
            status_info[ST_UNK] = []
            status_info[ST_IGN] = []
            info['filestatus'] = status_info

        return info

    def __str__(self):
        """String represenation"""
        return f'{self.__class__.__name__}({self.source_root})'


def load_vcs(name, directory, *args, **argv):
    """
    Load a specific vcs module for the given directory and arguments.

    Args:
        name: the vcs module name
        directory: the directory to load the vcs module for
    Raises:
        VCSUnsupported: if the vcs module does not support the given directory
        TypeError: if the vcs module does not support the given directory
    """
    LOGGER.debug(f'Loading VCS module: {name}')
    vcs_module = __import__('.'.join((__name__, name)), fromlist=[__name__])
    vcs = vcs_module.VCS(directory, *args, **argv)
    return vcs


def detect_vcss(directory, *args, **argv):
    """
    Interrogate the given directory for vcs information, returning an object
    that can be used to obtain information about the current conditions of the
    source tree.
    """
    directory = directory or os.getcwd()
    vcs_dir = os.path.dirname(__file__)
    # Ignore things that don't look like python modules
    # and list archive.py last.
    vcs_files = glob.glob(os.path.join(vcs_dir, '*.py'))
    possible_vcs = []
    archive_vcs = None

    errors = []
    for modpath in vcs_files:
        modname, _ = os.path.splitext(os.path.basename(modpath))
        # skip things like '__init__.py'
        if modname.startswith('_'):
            continue

        try:
            vcs = load_vcs(modname, directory, *args, **argv)
            if hasattr(vcs, 'is_archive') and vcs.is_archive:
                # if an "archive" vcs config is present, it always wins
                return [archive_vcs]

            possible_vcs.append(vcs)
        except (VCSUnsupported, TypeError) as err:
            LOGGER.debug(f'Failed {modname}: {err}')
            errors.append(str(err))

    if not possible_vcs:
        # pylint: disable=C0301
        message = f'No recognized VCS management of source tree "{directory}" - do you need to login to a VCS?'
        for error in errors:
            message += f'\n\tWARNING: {error}'
        raise VCSUnsupported(message)

    return possible_vcs


def detect_vcs(directory, *args, **argv):
    """
    Detects the VCS type for a given directory.
    :param directory: the directory to determine the VCS for
    :return: the VCS type
    """
    possible_vcss = detect_vcss(directory, *args, **argv)
    LOGGER.debug(f'Possible VCSs: {possible_vcss}', possible_vcss)

    if len(possible_vcss) > 1:
        possible_vcss_str = ', '.join([vcs.vcs for vcs in possible_vcss])
        LOGGER.warning(f'WARNING: Multiple VCS matches: {possible_vcss_str}')

    return possible_vcss[0]


try:
    import setuptools.command.egg_info

    # pyline: disable=R0904
    class VCSInfoEggInfo(setuptools.command.egg_info.egg_info):
        """
        Override the egg_info command to appropriately set build tags.
        """

        command_name = 'egg_info'

        def tags(self):
            sourcedir = os.path.dirname(
                os.path.abspath(self.distribution.script_name)
            )

            # first, see if we can get vcs information.
            try:
                vcs = detect_vcs(sourcedir)
                tag_build = f'.{vcs.number}'
                if vcs.modified > 0:
                    tag_build = f'{tag_build}.{vcs.modified}'
                return tag_build
            except TypeError:
                pass
            except VCSUnsupported:
                pass

            # if not fall back to the default routine
            return setuptools.command.egg_info.egg_info.tags(self)


except ImportError:
    pass


# Local Variables:
# fill-column: 100
# End:



import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'vcsinfo/VERSION')) as fobj:
    version = fobj.read().strip()

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as fobj:
    long_description = fobj.read().strip()

# pylint: disable=C0301
setup(
    name='vcsinfo',
    version=version,
    author='Adobe',
    author_email='noreply@adobe.com',
    license='MIT',
    url='https://github.com/adobe/vcsinfo',
    description='Utilities to normalize working with different Version Control Systems',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    packages=find_packages(),
    scripts=[
        'bin/vcsinfo',
    ],
    install_requires=[
        # This is the last version of GitPython that still works with Python 2.7
        'GitPython==2.1.8',
        # Newer versions of gitdb2 require Python 3.x - not everything has been pulled-up to 3.x
        'gitdb2<=2.0.6',
        'mercurial',
    ],
    extras_require={
        # Perforce is not as common anymore, so make it an optional dependency
        'p4': ['p4python'],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



"""
Copyright 2021 Adobe
All Rights Reserved.

NOTICE: Adobe permits you to use, modify, and distribute this file in accordance
with the terms of the Adobe license agreement accompanying it.
"""

from __future__ import absolute_import

import logging
import os
import vcsinfo

try:
    from mercurial import (
        hg,
        ui,
    )
except ImportError as impe:
    # pylint: disable=C0301
    raise vcsinfo.VCSUnsupported(f'Mercurial VCS module requires mercurial: {impe}')


LOGGER = logging.getLogger(__name__)


class VCSHg(vcsinfo.VCS):
    """Minimal class for querying a source tree managed by Mercurial (Hg)"""

    def __init__(self, dirname):
        vcsinfo.VCS.__init__(self)

        hgui = ui.ui()
        self.detect_source_root(dirname)

        try:
            # pylint: disable=C0301
            self.vcs_obj = hg.repository(hgui, path=self.source_root.encode('utf-8'), create=False)
        except TypeError as err:
            raise TypeError(f'Unable to initialize Hg: {err}') from err
        LOGGER.debug(f'Matched {self.vcs}: {dirname}')

    def detect_source_root(self, dirname):
        """Find the top-most source directory"""
        repo_dir = vcsinfo.search_parent_dirs(dirname, '.hg')
        if not repo_dir:
            raise TypeError(f"Directory '{dirname}' is not managed by hg")
        self.source_root = os.path.dirname(repo_dir)

    @property
    def upstream_repo(self):
        """The location of the up-stream VCS repository."""
        return self.vcs_obj.ui.config(b'paths', b'default')

    @property
    def name(self):
        if not self._name:
            if self.vcs_obj.ui.config(b'paths', b'default'):
                # There's an upstream - use the basename for the name
                # pylint: disable=C0301
                path = self.vcs_obj.ui.config(b'paths', b'default').decode('ascii')
            else:
                # No upstream - the directory is the repo - use the
                # directory basename (without "dot" extensions) as
                # the name.
                path = self.source_root
            self._name = os.path.splitext(os.path.basename(path))[0]
        return self._name

    @property
    def branch(self):
        return self.vcs_obj[b'.'].branch()

    @property
    def id(self):
        return self.vcs_obj[b'.'].hex()

    @property
    def id_short(self):
        return self.id[:6]

    @property
    def number(self):
        return int(self.vcs_obj[b'.'].rev())

    def status(self):
        files = self.vcs_obj.status(ignored=True, clean=True, unknown=True)
        status = []
        for fset in files:
            status.append([f.decode('ascii') for f in fset])
        return status

    def list_files(self):
        status = self.status()
        vcs_files = [f.decode('ascii') for f in
                     set(status[vcsinfo.ST_CLN])
                     | set(status[vcsinfo.ST_ADD])
                     | set(status[vcsinfo.ST_MOD])
                     ]
        vcs_files.sort()
        return vcs_files


VCS = VCSHg
